@@ -33,9 +33,25 @@ def mir_data_object():
 
 
 @pytest.fixture
-def uv_in_uvfits(tmp_path):
+def uv_in_ms(tmp_path):
     uv_in = UVData()
     testfile = os.path.join(DATA_PATH, "sma_test.mir")
+    write_file = os.path.join(tmp_path, "outtest_mir.ms")
+
+    # Currently only one source is supported.
+    uv_in.read(testfile)
+    uv_out = UVData()
+
+    yield uv_in, uv_out, write_file
+
+    # cleanup
+    del uv_in, uv_out
+
+
+@pytest.fixture
+def uv_in_uvfits(tmp_path):
+    uv_in = UVData()
+    testfile = os.path.join(DATA_PATH, "sma_test.mir/")
     write_file = os.path.join(tmp_path, "outtest_mir.uvfits")
 
     # Currently only one source is supported.
@@ -105,10 +121,109 @@ def test_read_mir_write_uvfits(uv_in_uvfits, future_shapes):
     assert 0 == uvfits_uv.history.find(
         mir_uv.history + "  Read/written with pyuvdata version:"
     )
-
     mir_uv.history = uvfits_uv.history
+
+    # We have to do a bit of special handling for the phase_center_catalog, because
+    # _very_ small errors (like last bit in the mantissa) creep in when passing through
+    # the util function transform_sidereal_coords (for mutli-phase-ctr datasets). Verify
+    # the two match up in terms of their coordinates
+    for cat_name in mir_uv.phase_center_catalog.keys():
+        assert np.isclose(
+            mir_uv.phase_center_catalog[cat_name]["cat_lat"],
+            uvfits_uv.phase_center_catalog[cat_name]["cat_lat"],
+        )
+        assert np.isclose(
+            mir_uv.phase_center_catalog[cat_name]["cat_lon"],
+            uvfits_uv.phase_center_catalog[cat_name]["cat_lon"],
+        )
+    uvfits_uv.phase_center_catalog = mir_uv.phase_center_catalog
+
+    # There's a minor difference between what SMA calculates online for app coords
+    # and what pyuvdata calculates, to the tune of ~1 arcsec. Check those values here,
+    # then set them equal to one another.
+    assert np.all(
+        np.abs(mir_uv.phase_center_app_ra - uvfits_uv.phase_center_app_ra) < 1e-5
+    )
+
+    assert np.all(
+        np.abs(mir_uv.phase_center_app_dec - uvfits_uv.phase_center_app_dec) < 1e-5
+    )
+
+    mir_uv._set_app_coords_helper()
+    uvfits_uv._set_app_coords_helper()
+
+    # make sure filenames are what we expect
+    assert mir_uv.filename == ["sma_test.mir"]
+    assert uvfits_uv.filename == ["outtest_mir.uvfits"]
+    mir_uv.filename = uvfits_uv.filename
     assert mir_uv == uvfits_uv
 
+    # Since mir is mutli-phase-ctr by default, this should effectively be a no-op
+    mir_uv._set_multi_phase_center()
+
+    assert mir_uv == uvfits_uv
+
+
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
+@pytest.mark.filterwarnings("ignore:LST values stored in this file are not ")
+@pytest.mark.parametrize("future_shapes", [True, False])
+def test_read_mir_write_ms(uv_in_ms, future_shapes):
+    """
+    Mir to uvfits loopback test.
+
+    Read in Mir files, write out as ms, read back in and check for
+    object equality.
+    """
+    pytest.importorskip("casacore")
+    mir_uv, ms_uv, testfile = uv_in_ms
+
+    if future_shapes:
+        mir_uv.use_future_array_shapes()
+
+    mir_uv.write_ms(testfile, clobber=True)
+    ms_uv.read(testfile)
+
+    # Single integration with 1 phase center = single scan number
+    # output in the MS
+    assert ms_uv.scan_number_array == np.array([1])
+
+    if future_shapes:
+        ms_uv.use_future_array_shapes()
+
+    # There are some minor differences between the values stored by MIR and that
+    # calculated by UVData. Since MS format requires these to be calculated on the fly,
+    # we calculate them here just to verify that everything is looking okay.
+    mir_uv.set_lsts_from_time_array()
+    mir_uv._set_app_coords_helper()
+
+    # These reorderings just make sure that data from the two formats are lined up
+    # correctly.
+    mir_uv.reorder_freqs(spw_order="number")
+    ms_uv.reorder_blts()
+
+    # MS doesn't have the concept of an "instrument" name like FITS does, and instead
+    # defaults to the telescope name. Make sure that checks out here.
+    assert mir_uv.instrument == "SWARM"
+    assert ms_uv.instrument == "SMA"
+    mir_uv.instrument = ms_uv.instrument
+
+    # Quick check for history here
+    assert ms_uv.history != mir_uv.history
+    ms_uv.history = mir_uv.history
+
+    # Only MS has extra keywords, verify those look as expected.
+    assert ms_uv.extra_keywords == {"DATA_COL": "DATA", "observer": "SMA"}
+    assert mir_uv.extra_keywords == {}
+    mir_uv.extra_keywords = ms_uv.extra_keywords
+
+    # Make sure the filenames line up as expected.
+    assert mir_uv.filename == ["sma_test.mir"]
+    assert ms_uv.filename == ["outtest_mir.ms"]
+    mir_uv.filename = ms_uv.filename = None
+
+    # Finally, with all exceptions handled, check for equality.
+    assert ms_uv.__eq__(mir_uv, allowed_failures=["filename"])
+
 
 @pytest.mark.filterwarnings("ignore:LST values stored ")
 def test_read_mir_write_uvh5(uv_in_uvh5):
@@ -131,6 +246,11 @@ def test_read_mir_write_uvh5(uv_in_uvh5):
     # test fails because of updated history, so this is our workaround for now.
     mir_uv.history = uvh5_uv.history
 
+    # make sure filenames are what we expect
+    assert mir_uv.filename == ["sma_test.mir"]
+    assert uvh5_uv.filename == ["outtest_mir.uvh5"]
+    mir_uv.filename = uvh5_uv.filename
+
     assert mir_uv == uvh5_uv
 
 
@@ -139,7 +259,7 @@ def test_write_mir(uv_in_uvfits, err_type=NotImplementedError):
     Mir writer test
 
     Check and make sure that attempts to use the writer return a
-    'not implemented; error.
+    'not implemented' error.
     """
     mir_uv, uvfits_uv, testfile = uv_in_uvfits
 
@@ -172,9 +292,12 @@ def test_read_mir_no_records():
     """
     testfile = os.path.join(DATA_PATH, "sma_test.mir")
     uv_in = UVData()
-    with pytest.raises(IndexError, match="No valid records matching those selections!"):
+    with pytest.raises(IndexError, match="No valid sources selected!"):
         uv_in.read_mir(testfile, isource=-1)
 
+    with pytest.raises(IndexError, match="No valid records matching those selections!"):
+        uv_in.read_mir(testfile, irec=-1)
+
     with pytest.raises(IndexError, match="No valid sidebands selected!"):
         uv_in.read_mir(testfile, isb=[])
 
@@ -182,6 +305,36 @@ def test_read_mir_no_records():
         uv_in.read_mir(testfile, isb=[-156])
 
 
+def test_read_mir_sideband_select():
+    """
+    Mir sideband read check
+
+    Make sure that we can read the individual sidebands out of MIR correctly, and then
+    stitch them back together as though they were read together from the start.
+    """
+    testfile = os.path.join(DATA_PATH, "sma_test.mir")
+    mir_dsb = UVData()
+    mir_dsb.read(testfile)
+    # Re-order here so that we can more easily compare the two
+    mir_dsb.reorder_freqs(channel_order="freq", spw_order="freq")
+    # Drop the history
+    mir_dsb.history = ""
+
+    mir_lsb = UVData()
+    mir_lsb.read(testfile, isb=[0])
+
+    mir_usb = UVData()
+    mir_usb.read(testfile, isb=[1])
+
+    mir_recomb = mir_lsb + mir_usb
+    # Re-order here so that we can more easily compare the two
+    mir_recomb.reorder_freqs(spw_order="freq", channel_order="freq")
+    # Drop the history
+    mir_recomb.history = ""
+
+    assert mir_dsb == mir_recomb
+
+
 def test_mir_auto_read(
     err_type=IndexError, err_msg="Could not determine auto-correlation record size!"
 ):
@@ -191,7 +344,7 @@ def test_mir_auto_read(
     Make sure that Mir autocorrelations are read correctly
     """
     testfile = os.path.join(DATA_PATH, "sma_test.mir")
-    mir_data = mir_parser.MirParser(testfile)
+    mir_data = mir_parser.MirParser(testfile, has_auto=True)
     with pytest.raises(err_type, match=err_msg):
         ac_data = mir_data.scan_auto_data(testfile, nchunks=999)
 
@@ -228,7 +381,13 @@ def test_mir_remember_me_record_lengths(mir_data_object):
     mir_data = mir_data_object
 
     # Check to make sure we've got the right number of records everywhere
-    assert len(mir_data.ac_read) == 2
+
+    # ac_read only exists if has_auto=True
+    if mir_data.ac_read is not None:
+        assert len(mir_data.ac_read) == 2
+    else:
+        # This should only occur when has_auto=False
+        assert not mir_data._has_auto
 
     assert len(mir_data.bl_read) == 4
 
@@ -418,19 +577,27 @@ def test_mir_remember_me_ac_read(mir_data_object):
     mir_data = mir_data_object
 
     # Now check ac_read
-    assert np.all(mir_data.ac_read["inhid"] == 1)
 
-    assert np.all(mir_data.ac_read["achid"] == np.arange(1, 3))
+    # ac_read only exists if has_auto=True
+    if mir_data.ac_read is not None:
 
-    assert np.all(mir_data.ac_read["antenna"] == [1, 4])
+        assert np.all(mir_data.ac_read["inhid"] == 1)
 
-    assert np.all(mir_data.ac_read["nchunks"] == 8)
+        assert np.all(mir_data.ac_read["achid"] == np.arange(1, 3))
 
-    assert np.all(mir_data.ac_read["datasize"] == 1048596)
+        assert np.all(mir_data.ac_read["antenna"] == [1, 4])
 
-    assert np.all(mir_data.we_read["scanNumber"] == 1)
+        assert np.all(mir_data.ac_read["nchunks"] == 8)
 
-    assert np.all(mir_data.we_read["flags"] == 0)
+        assert np.all(mir_data.ac_read["datasize"] == 1048596)
+
+        assert np.all(mir_data.we_read["scanNumber"] == 1)
+
+        assert np.all(mir_data.we_read["flags"] == 0)
+
+    else:
+        # This should only occur when has_auto=False
+        assert not mir_data._has_auto
 
 
 def test_mir_remember_me_sp_read(mir_data_object):
