@@ -77,6 +77,45 @@ def test_read_nrao(casa_uvfits):
     assert uvobj2 == uvobj3
 
 
+@pytest.mark.filterwarnings("ignore:ITRF coordinate frame detected")
+@pytest.mark.filterwarnings("ignore:Telescope OVRO_MMA is not")
+@pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
+def test_time_precision(tmp_path):
+    """
+    This tests that the times are round-tripped through write/read uvfits to sufficient
+    precision.
+    """
+    pytest.importorskip("casacore")
+    lwa_file = os.path.join(
+        DATA_PATH, "2018-03-21-01_26_33_0004384620257280_000000_downselected.ms"
+    )
+    uvd = UVData()
+    uvd.read(lwa_file)
+
+    testfile = os.path.join(tmp_path, "lwa_testfile.uvfits")
+    uvd.write_uvfits(testfile, spoof_nonessential=True)
+
+    uvd2 = UVData()
+    uvd2.read(testfile)
+
+    latitude, longitude, altitude = uvd2.telescope_location_lat_lon_alt_degrees
+    unique_times, inverse_inds = np.unique(uvd2.time_array, return_inverse=True)
+    unique_lst_array = uvutils.get_lst_for_time(
+        unique_times, latitude, longitude, altitude,
+    )
+
+    calc_lst_array = unique_lst_array[inverse_inds]
+
+    assert np.allclose(
+        calc_lst_array,
+        uvd2.lst_array,
+        rtol=uvd2._lst_array.tols[0],
+        atol=uvd2._lst_array.tols[1],
+    )
+
+    assert uvd2.__eq__(uvd, allowed_failures=["filename", "scan_number_array"])
+
+
 @pytest.mark.filterwarnings("ignore:Telescope EVLA is not")
 def test_break_read_uvfits():
     """Test errors on reading in a uvfits file with subarrays and other problems."""
@@ -107,11 +146,23 @@ def test_source_group_params(casa_uvfits, tmp_path):
         par_names = vis_hdu.data.parnames
         group_parameter_list = []
 
+        time_ind = 0
         lst_ind = 0
         for index, name in enumerate(par_names):
             par_value = vis_hdu.data.par(name)
-            # lst_array needs to be split in 2 parts to get high enough accuracy
-            if name.lower() == "lst":
+            # time_array and lst_array need to be split in 2 parts to get high enough
+            # accuracy
+            if name.lower() == "date":
+                if time_ind == 0:
+                    # first time entry, par_value has full time value
+                    # (astropy adds the 2 values)
+                    time_array1 = np.float32(par_value)
+                    time_array2 = np.float32(par_value - np.float64(time_array1))
+                    par_value = time_array1
+                    time_ind = 1
+                else:
+                    par_value = time_array2
+            elif name.lower() == "lst":
                 if lst_ind == 0:
                     # first lst entry, par_value has full lst value
                     # (astropy adds the 2 values)
@@ -171,11 +222,24 @@ def test_source_frame_defaults(casa_uvfits, tmp_path):
         par_names = vis_hdu.data.parnames
         group_parameter_list = []
 
+        time_ind = 0
         lst_ind = 0
         for index, name in enumerate(par_names):
             par_value = vis_hdu.data.par(name)
             # lst_array needs to be split in 2 parts to get high enough accuracy
-            if name.lower() == "lst":
+            # time_array and lst_array need to be split in 2 parts to get high enough
+            # accuracy
+            if name.lower() == "date":
+                if time_ind == 0:
+                    # first time entry, par_value has full time value
+                    # (astropy adds the 2 values)
+                    time_array1 = np.float32(par_value)
+                    time_array2 = np.float32(par_value - np.float64(time_array1))
+                    par_value = time_array1
+                    time_ind = 1
+                else:
+                    par_value = time_array2
+            elif name.lower() == "lst":
                 if lst_ind == 0:
                     # first lst entry, par_value has full lst value
                     # (astropy adds the 2 values)
@@ -225,11 +289,23 @@ def test_missing_aips_su_table(casa_uvfits, tmp_path):
         par_names = vis_hdu.data.parnames
         group_parameter_list = []
 
+        time_ind = 0
         lst_ind = 0
         for index, name in enumerate(par_names):
             par_value = vis_hdu.data.par(name)
-            # lst_array needs to be split in 2 parts to get high enough accuracy
-            if name.lower() == "lst":
+            # time_array and lst_array need to be split in 2 parts to get high enough
+            # accuracy
+            if name.lower() == "date":
+                if time_ind == 0:
+                    # first time entry, par_value has full time value
+                    # (astropy adds the 2 values)
+                    time_array1 = np.float32(par_value)
+                    time_array2 = np.float32(par_value - np.float64(time_array1))
+                    par_value = time_array1
+                    time_ind = 1
+                else:
+                    par_value = time_array2
+            elif name.lower() == "lst":
                 if lst_ind == 0:
                     # first lst entry, par_value has full lst value
                     # (astropy adds the 2 values)
@@ -359,6 +435,157 @@ def test_readwriteread(tmp_path, casa_uvfits, future_shapes):
     return
 
 
+@pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
+@pytest.mark.filterwarnings("ignore:Telescope EVLA is not in known_telescopes.")
+@pytest.mark.parametrize("uvw_suffix", ["---SIN", "---NCP"])
+def test_uvw_coordinate_suffixes(casa_uvfits, tmp_path, uvw_suffix):
+    uv_in = casa_uvfits
+    write_file = str(tmp_path / "outtest_casa.uvfits")
+    write_file2 = str(tmp_path / "outtest_casa2.uvfits")
+    uv_in.write_uvfits(write_file)
+
+    with fits.open(write_file, memmap=True) as hdu_list:
+        hdunames = uvutils._fits_indexhdus(hdu_list)
+        vis_hdu = hdu_list[0]
+        vis_hdr = vis_hdu.header.copy()
+        raw_data_array = vis_hdu.data.data
+
+        par_names = vis_hdu.data.parnames
+        group_parameter_list = []
+
+        time_ind = 0
+        lst_ind = 0
+        for index, name in enumerate(par_names):
+            par_value = vis_hdu.data.par(name)
+            # time_array and lst_array need to be split in 2 parts to get high enough
+            # accuracy
+            if name.lower() == "date":
+                if time_ind == 0:
+                    # first time entry, par_value has full time value
+                    # (astropy adds the 2 values)
+                    time_array1 = np.float32(par_value)
+                    time_array2 = np.float32(par_value - np.float64(time_array1))
+                    par_value = time_array1
+                    time_ind = 1
+                else:
+                    par_value = time_array2
+            elif name.lower() == "lst":
+                if lst_ind == 0:
+                    # first lst entry, par_value has full lst value
+                    # (astropy adds the 2 values)
+                    lst_array_1 = np.float32(par_value)
+                    lst_array_2 = np.float32(par_value - np.float64(lst_array_1))
+                    par_value = lst_array_1
+                    lst_ind = 1
+                else:
+                    par_value = lst_array_2
+
+            # need to account for PZERO values
+            group_parameter_list.append(par_value - vis_hdr["PZERO" + str(index + 1)])
+
+        for name in ["UU", "VV", "WW"]:
+            par_names[par_names.index(name)] = name + uvw_suffix
+        vis_hdu = fits.GroupData(
+            raw_data_array, parnames=par_names, pardata=group_parameter_list, bitpix=-32
+        )
+        vis_hdu = fits.GroupsHDU(vis_hdu)
+        vis_hdu.header = vis_hdr
+        ant_hdu = hdu_list[hdunames["AIPS AN"]]
+
+        hdulist = fits.HDUList(hdus=[vis_hdu, ant_hdu])
+        hdulist.writeto(write_file2, overwrite=True)
+        hdulist.close()
+
+    if uvw_suffix == "---NCP":
+        with uvtest.check_warnings(
+            UserWarning,
+            match=[
+                "Telescope EVLA is not in known_telescopes.",
+                "The baseline coordinates (uvws) in this file are specified in the "
+                "---NCP coordinate system",
+                "The uvw_array does not match the expected values",
+            ],
+        ):
+            uv2 = UVData.from_file(write_file2)
+        uv2.uvw_array = uvutils._rotate_one_axis(
+            uv2.uvw_array[:, :, None], -1 * (uv2.phase_center_app_dec - np.pi / 2), 0
+        )[:, :, 0]
+    else:
+        uv2 = UVData.from_file(write_file2)
+
+    assert uv2 == uv_in
+
+
+@pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
+@pytest.mark.filterwarnings("ignore:Telescope EVLA is not in known_telescopes.")
+@pytest.mark.parametrize(
+    "uvw_suffix", [["---SIN", "", ""], ["", "---NCP", ""], ["", "---NCP", "---SIN"]]
+)
+def test_uvw_coordinate_suffix_errors(casa_uvfits, tmp_path, uvw_suffix):
+    uv_in = casa_uvfits
+    write_file = str(tmp_path / "outtest_casa.uvfits")
+    write_file2 = str(tmp_path / "outtest_casa2.uvfits")
+    uv_in.write_uvfits(write_file)
+
+    with fits.open(write_file, memmap=True) as hdu_list:
+        hdunames = uvutils._fits_indexhdus(hdu_list)
+        vis_hdu = hdu_list[0]
+        vis_hdr = vis_hdu.header.copy()
+        raw_data_array = vis_hdu.data.data
+
+        par_names = vis_hdu.data.parnames
+        group_parameter_list = []
+
+        time_ind = 0
+        lst_ind = 0
+        for index, name in enumerate(par_names):
+            par_value = vis_hdu.data.par(name)
+            # time_array and lst_array need to be split in 2 parts to get high enough
+            # accuracy
+            if name.lower() == "date":
+                if time_ind == 0:
+                    # first time entry, par_value has full time value
+                    # (astropy adds the 2 values)
+                    time_array1 = np.float32(par_value)
+                    time_array2 = np.float32(par_value - np.float64(time_array1))
+                    par_value = time_array1
+                    time_ind = 1
+                else:
+                    par_value = time_array2
+            elif name.lower() == "lst":
+                if lst_ind == 0:
+                    # first lst entry, par_value has full lst value
+                    # (astropy adds the 2 values)
+                    lst_array_1 = np.float32(par_value)
+                    lst_array_2 = np.float32(par_value - np.float64(lst_array_1))
+                    par_value = lst_array_1
+                    lst_ind = 1
+                else:
+                    par_value = lst_array_2
+
+            # need to account for PZERO values
+            group_parameter_list.append(par_value - vis_hdr["PZERO" + str(index + 1)])
+
+        for ind, name in enumerate(["UU", "VV", "WW"]):
+            par_names[par_names.index(name)] = name + uvw_suffix[ind]
+        vis_hdu = fits.GroupData(
+            raw_data_array, parnames=par_names, pardata=group_parameter_list, bitpix=-32
+        )
+        vis_hdu = fits.GroupsHDU(vis_hdu)
+        vis_hdu.header = vis_hdr
+        ant_hdu = hdu_list[hdunames["AIPS AN"]]
+
+        hdulist = fits.HDUList(hdus=[vis_hdu, ant_hdu])
+        hdulist.writeto(write_file2, overwrite=True)
+        hdulist.close()
+
+    with pytest.raises(
+        ValueError,
+        match="There is no consistent set of baseline coordinates in this file.",
+    ):
+        UVData.from_file(write_file2)
+
+
 @pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
 @pytest.mark.filterwarnings("ignore:Telescope EVLA is not")
 def test_readwriteread_no_lst(tmp_path, casa_uvfits):
@@ -445,7 +672,9 @@ def test_readwriteread_large_antnums(tmp_path, casa_uvfits):
         [
             "The uvw_array does not match the expected values given the antenna "
             "positions",
-            "antnums_to_baseline: found > 256 antennas, using 2048 baseline",
+            "Found antenna numbers > 256 in this data set. This is permitted by "
+            "UVFITS ",
+            "antnums_to_baseline: found antenna numbers > 256, using 2048 baseline",
         ],
     ):
         uv_in.write_uvfits(write_file)
@@ -551,22 +780,27 @@ def test_readwriteread_error_single_time(tmp_path, casa_uvfits):
         hdulist = fits.HDUList(hdus=[vis_hdu, ant_hdu])
         hdulist.writeto(write_file2, overwrite=True)
 
-    with pytest.raises(ValueError) as cm:
+    with pytest.raises(
+        ValueError, match="Required UVParameter _integration_time has not been set"
+    ):
         with uvtest.check_warnings(
-            [UserWarning, erfa.core.ErfaWarning, erfa.core.ErfaWarning, UserWarning],
+            [
+                UserWarning,
+                erfa.core.ErfaWarning,
+                erfa.core.ErfaWarning,
+                UserWarning,
+                UserWarning,
+            ],
             [
                 "Telescope EVLA is not",
-                'ERFA function "utcut1" yielded 1 of "dubious year (Note 3)"',
-                'ERFA function "utctai" yielded 1 of "dubious year (Note 3)"',
+                "ERFA function 'utcut1' yielded 1 of 'dubious year (Note 3)'",
+                "ERFA function 'utctai' yielded 1 of 'dubious year (Note 3)'",
                 "LST values stored in this file are not self-consistent",
+                "The integration time is not specified and only one time",
             ],
         ):
             uv_out.read(write_file2),
 
-    assert str(cm.value).startswith(
-        "integration time not specified and only one time present"
-    )
-
     return
 
 
@@ -771,7 +1005,7 @@ def test_select_read(casa_uvfits, tmp_path, select_kwargs):
     assert uvfits_uv == uvfits_uv2
 
 
-@pytest.mark.filterwarnings("ignore:Required Antenna frame keyword")
+@pytest.mark.filterwarnings("ignore:Required Antenna keyword 'FRAME'")
 @pytest.mark.filterwarnings("ignore:telescope_location is not set")
 @pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
 @pytest.mark.parametrize(
@@ -1179,6 +1413,9 @@ def test_read_ms_write_uvfits_casa_history(tmp_path):
     assert uvfits_uv.filename == ["outtest.uvfits"]
     ms_uv.filename = uvfits_uv.filename
 
+    # propagate scan numbers to the uvfits, ONLY for comparison
+    uvfits_uv.scan_number_array = ms_uv.scan_number_array
+
     assert ms_uv == uvfits_uv
 
 
@@ -1199,10 +1436,7 @@ def test_cotter_telescope_frame(tmp_path):
 
     with uvtest.check_warnings(
         UserWarning,
-        [
-            "Required Antenna frame keyword not set, but this appears to be a Cotter "
-            "file, setting to ITRF.",
-        ],
+        ["Required Antenna keyword 'FRAME' not set; Assuming frame is 'ITRF'."],
     ):
         uvd1.read_uvfits(write_file, read_data=False)
 
@@ -1258,3 +1492,32 @@ def test_flex_spw_uvfits_write_errs(sma_mir, freq_val, chan_val, msg):
     sma_mir.channel_width[:] = chan_val
     with pytest.raises(ValueError, match=msg):
         sma_mir.write_uvfits("dummy", spoof_nonessential=True)
+
+
+def test_mwax_birli_frame(tmp_path):
+    fits_file = os.path.join(DATA_PATH, "1061316296.uvfits")
+    outfile = tmp_path / "mwax_birli.uvfits"
+    with fits.open(fits_file, memmap=True) as hdu_list:
+        hdu_list[0].header["SOFTWARE"] = "birli"
+        # remove the frame keyword
+        del hdu_list[1].header["FRAME"]
+        hdu_list.writeto(outfile)
+    with uvtest.check_warnings(
+        UserWarning,
+        ["Required Antenna keyword 'FRAME' not set; Assuming frame is 'ITRF'."],
+    ):
+        UVData.from_file(outfile, read_data=False)
+
+
+def test_mwax_missing_frame_comment(tmp_path):
+    fits_file = os.path.join(DATA_PATH, "1061316296.uvfits")
+    outfile = tmp_path / "mwax_birli.uvfits"
+    with fits.open(fits_file, memmap=True) as hdu_list:
+        del hdu_list[1].header["FRAME"], hdu_list[0].header["COMMENT"]
+        hdu_list[0].header["COMMENT"] = "A dummy comment."
+        hdu_list.writeto(outfile)
+    with uvtest.check_warnings(
+        UserWarning,
+        ["Required Antenna keyword 'FRAME' not set; Assuming frame is 'ITRF'."],
+    ):
+        UVData.from_file(outfile, read_data=False)
